<?php

// https://forum.qnapclub.de/blog/entry/456-tanze-samba-mit-pi-teil-5-einrichtung-des-active-directory/
// https://wiki.ubuntuusers.de/Howto/Samba-AD_Tipps/
// https://wiki.ubuntuusers.de/HowTo/Samba-AD-Server_unter_Ubuntu_20.04_installieren/
// https://github.com/Jean28518/linux-guides/tree/main/samba-dc
// https://www.considerednormal.com/2022/11/samba-based-active-directory-on-ubuntu-22-04/
// https://medium.com/@techbeatz/active-directory-how-to-join-a-linux-ubuntu-client-machine-into-an-active-directory-domain-734d0bbd19bd
// https://github.com/Jean28518/linux-guides/tree/main/join-samba-dc-as-client



# Server
# /etc/samba/smb.conf
# Auskommentieren
#dns forwarder resolve only = yes


/*
# https://github.com/Jean28518/linux-guides/tree/main/samba-dc

su -
hostnamectl set-hostname la

echo "192.168.1.171 la.int.de la" >> /etc/hosts # IP of the server itself

# Only on ubuntu:
systemctl disable --now systemd-resolved
unlink /etc/resolv.conf

# Make sure you are not running any dns server 
apt purge -y dnsmasq

echo "# OpenDNS
nameserver 208.67.222.222
# Samba server IP address
nameserver 192.168.1.171
# fallback
nameserver 8.8.8.8
# main domain for Samba
search int.de" > /etc/resolv.conf

# Make /etc/resolv.conf immutable because the system sometimes overwrites it.
chattr +i /etc/resolv.conf
apt update && apt install -y acl attr samba samba-dsdb-modules samba-vfs-modules smbclient winbind libpam-winbind libnss-winbind libpam-krb5 krb5-config krb5-user dnsutils chrony net-tools samba-ad-provision
# Default Keberos Realm: INT.DE
# Kerberos Server for your realm: la.int.de
# Administrations server for your realm: la.int.de

systemctl disable --now smbd nmbd winbind
systemctl unmask samba-ad-dc
systemctl enable samba-ad-dc

mv /etc/samba/smb.conf /etc/samba/smb.conf.bak
samba-tool domain provision
# Realm: INT.DE
# Domain: INT
# Server Role: dc
# Backend: SAMBA_INTERNAL
# DNS forwarder IP adress: 208.67.222.222
# Administrator password: Gae7Eexo # Or take a password without bash special characters!

mv /etc/krb5.conf /etc/krb5.conf.orig
cp /var/lib/samba/private/krb5.conf /etc/krb5.conf

systemctl restart samba-ad-dc

samba-tool domain passwordsettings set --complexity=off
samba-tool domain passwordsettings set --history-length=0
samba-tool domain passwordsettings set --min-pwd-age=0
samba-tool domain passwordsettings set --max-pwd-age=0


cd /etc/samba/tls/
openssl req -newkey rsa:2048 -keyout myKey.pem -nodes -x509 -days 3650 -out myCert.pem
chmod 600 myKey.pem
mkdir -p /var/www/cert
cp myCert.pem /var/www/cert/samba.crt

nano /etc/samba/smb.conf
# Add in [global]
        tls enabled  = yes
        tls keyfile  = /etc/samba/tls/myKey.pem
        tls certfile = /etc/samba/tls/myCert.pem
        tls cafile   =

systemctl restart samba-ad-dc.service
ufw allow ldaps

apt install -y ldap-utils
*/



class CLibreWorkspace extends CClient
{

	private
		$domain = '',
		$admin_password = '',
		$ldap_dc = '',
		$realm = '',
		$client = '';





/**
**name CLibreWorkspace::__construct($client)
**description Constructor for new CLibreWorkspace objects. The object holds all information about the partitioning (of a client and loads the values from the DB).
**parameter client: Name of an existing client to load.
**/
	public function __construct($client)
	{
		parent::__construct($client, CClient::CHECKMODE_MUSTEXIST);
		CLibreWorkspace::parseSettingsFile($this->domain, $this->admin_password, $this->ldap_dc, $this->realm);
		$this->client = $client;
	}





/**
**name CLibreWorkspace::__destruct()
**description Destructor for a CLibreWorkspace object. Before the object is removed from the RAM, all settings are written to the DB.
**/
	function __destruct()
	{
		parent::__destruct();
	}





/**
**name CLibreWorkspace::parseSettingsFile(&$domain, &$admin_password, &$ldap_dc, &$realm)
**description Parses the Libre Workspace settings file that is written during m23 installation.
**parameter domain: The main domain (eg int.de) of Libre Workspace.
**parameter admin_password: Libre Workspace's administrator password.
**parameter ldap_dc: Libre Workspace's LDAP DC.
**parameter realm: Get Libre Workspace's Kerberos Realm.
**returns CLibreWorkspace object on success, otherwise false.
**/
	public static function parseSettingsFile(&$domain, &$admin_password, &$ldap_dc, &$realm)
	{
		$lines = file('/m23/etc/lw.settings');

		// Parse the config file line by line
		foreach ($lines as $line)
		{
			// Remove newlines
			$line = trim($line);

			// Separate setting name from value
			$varVal = explode(': ', $line);

			switch($varVal[0])
			{
				case 'DOMAIN': $domain = $varVal[1]; break;
				case 'ADMIN_PASSWORD': $admin_password = $varVal[1]; break;
				case 'LDAP_DC': $ldap_dc = $varVal[1]; break;
			}
		}
		
		$realm = str_replace('dc=','',$ldap_dc);
		$realm = str_replace(',','.',$realm);
		$realm = strtoupper($realm);
	}





/**
**name CLibreWorkspace::createCLibreWorkspaceObject(&$errorMsg)
**description Tries to create a CLibreWorkspace object. This only works if there m23 is installed on Libre Workspace.
**parameter client: Name of the client the 
**parameter errorMsg: In case of an error the error message is written into this variable.
**returns CLibreWorkspace object on success, otherwise false.
**/
	public static function createCLibreWorkspaceObject($client, &$errorMsg)
	{
		if (!CLibreWorkspace::isAvailable()) return(false);
	
		try
		{
			$CLibreWorkspaceO = new CLibreWorkspace($client);
		}
		catch (Exception $e)
		{
			$errorMsg = $e->getMessage();
		}

		return(isset($CLibreWorkspaceO) && is_object($CLibreWorkspaceO) ? $CLibreWorkspaceO : false);
	}





/**
**name CLibreWorkspace::addTempAdmin($client)
**description Creates a temporary Samba admin account that can be used to join an m23 client to Samba's AD.
**parameter client: Name of a client.
**returns Temporary password in success or false in case of error
**/
	public static function addTempAdmin($client)
	{
		CLibreWorkspace::delTempAdmin($client);

		$admin = CLibreWorkspace::getTempAdminName($client);
	
		// Generate a password that is random enough for Samba
		$passwd = trim(shell_exec('pwgen -1 -c -n -s 20'));
	
		// Create the admin account and add it to the groups
		$ret = SERVER_runInBackground('LW_addOTPAdmin',
		"
			ret=0
			# Create a new admin user (for joins)
			samba-tool user create '$admin' '$passwd'
			ret=$[ \$ret + \$? ]
	
			# Give admin rights
			samba-tool group addmembers Administrators '$admin'
			ret=$[ \$ret + \$? ]
			samba-tool group addmembers 'Domain Admins' '$admin'
			ret=$[ \$ret + \$? ]
			samba-tool group addmembers 'Enterprise Admins' '$admin'
			ret=$[ \$ret + \$? ]
			echo \$ret
		", 'root', false);
	
		// Split the output of the commands into lines
		$lines = explode("\n", $ret);
	
		// Check all lines if they contain hints for successfully creation
		$ok = (HELPER_str_contains($lines[0], 'added successfully')) &&
			(HELPER_str_contains($lines[1], 'Added members to group')) &&
			(HELPER_str_contains($lines[2], 'Added members to group')) &&
			(HELPER_str_contains($lines[3], 'Added members to group')) &&
			($lines[4] == 0);
	
		// Give back the temporary password in success  or false in case of error
		return($ok ? $passwd : false);
	}





/**
**name CLibreWorkspace::delTempAdmin($client)
**description Deletes a temporary Samba admin account.
**parameter client: Name of a client.
**returns true, if the accound was deleted successfully, otherwise false.
**/
	public static function delTempAdmin($client)
	{
		$admin = CLibreWorkspace::getTempAdminName($client);
	
		$ret = SERVER_runInBackground('LW_delTempAdmin',"samba-tool user delete $admin; echo \$?", 'root', false);
		
		// Split the output of the commands into lines
		$lines = explode("\n", $ret);
	
		// Check all lines if they contain hints for successfully deletion
		$ok = (strpos($lines[0], 'Deleted user') === 0) &&	// strpos: ok
			($lines[1] == 0);
	
		return($ok);
	}





/**
**name CLibreWorkspace::isAvailable()
**description Checks, if the m23 server is installed on LibreWorkspace.
**returns true, the m23 server is installed on LibreWorkspace, otherwise false.
**/
	public static function isAvailable()
	{
		// Same check in /m23/bin/isLibreworkspace
		return(file_exists('/usr/share/linux-arbeitsplatz'));
	}





/**
**name CLibreWorkspace::addUser($login, $firstName, $lastName, $password, $shell = '/bin/bash')
**description Creates a user account in Samba.
**parameter login: Login name
**parameter firstName: Forename
**parameter lastName: Familyname
**parameter password: Password
**parameter shell: Shell to use when logged in
**returns true, if the user was added successfully, otherwise (error) messages from samba-tool.
**/
	public static function addUser($login, $firstName, $lastName, $password, $shell = '/bin/bash')
	{
		$ret = SERVER_runInBackground('LW_addPosix',
		"
			ret=0
			#--uid-number= --gid-number=

			# Create a new admin user (for joins)
			samba-tool user create '$login' '$password' --given-name='$firstName' --surname='$lastName' --login-shell='$shell'
			ret=$[ \$ret + \$? ]

			#samba-tool group add '\$login' ---gid-number=\$gid
			#ret=$[ \$ret + \$? ]

			# Default user groups?
			samba-tool group addmembers users '$login'
			ret=$[ \$ret + \$? ]

			echo \$ret
		", 'root', false, true);

		// Split the output of the commands into lines
		$lines = explode("\n", $ret);

		// Check all lines if they contain hints for successfully deletion
		$ok = (HELPER_str_contains($lines[0], 'added successfully')) &&
			(HELPER_str_contains($lines[1], 'Added members to group')) &&
			($lines[2] == 0);

		return($ok ? true : $ret);
// 		DB_addNewUserID($uid);
// 		DB_addNewGroupID($gid);
	}





/**
**name CLibreWorkspace::getDomain()
**description The main domain (eg int.de) of Libre Workspace.
**returns Main domain (eg int.de) of Libre Workspace.
**/
	private function getDomain()
	{
		return($this->domain);
	}





/**
**name CLibreWorkspace::getRealm()
**description Get Libre Workspace's Kerberos Realm.
**returns Libre Workspace's Kerbers Realm.
**/
	private function getRealm()
	{
		return($this->realm);
	}





/**
**name CLibreWorkspace::getAdminPassword()
**description Get Libre Workspace's administrator password.
**returns Libre Workspace's administrator password.
**/
	private function getAdminPassword()
	{
		return($this->admin_password);
	}





/**
**name CLibreWorkspace::getLDAP_DC()
**description Get Libre Workspace's LDAP DC.
**returns Libre Workspace's LDAP DC.
**/
	private function getLDAP_DC()
	{
		return($this->ldap_dc);
	}





/**
**name CLibreWorkspace::getTempAdminName($client)
**description Get user name of a temporary admin account for a given client.
**parameter client: Name of a client.
**returns Name of a temporary admin account for a given client.
**/
	public static function getTempAdminName($client)
	{
		return("LW$client");
	}





/**
**name CLibreWorkspace::joinClientBASH()
**description Generates BASH code to run on the client to join the Libre Workspace Samba AD.
**returns BASH code to run on the client to join the Libre Workspace Samba AD.
**/
	public function joinClientBASH()
	{
		$DOMAIN = CLibreWorkspace::getDomain();
		$REALM = CLibreWorkspace::getRealm();
		$bulkPass = $this->getLibreWorkspaceTempAdmin();
		$client = $this->getClientname();
		$admin = CLibreWorkspace::getTempAdminName($client);
		$IP = CLibreWorkspace::subdomainOrIP();

		// BASH code for joining
		echo("
			apt-get update -y
			apt-get install -y sssd-ad sssd-tools realmd adcli packagekit
		
			echo '[libdefaults]
default_domain = $REALM
default_realm = $REALM
rdns = false' > /etc/krb5.conf

			apt-get install -y krb5-user sssd-krb5

			hostnamectl set-hostname $client.$DOMAIN

			yes '$bulkPass' | kinit $admin

			# yes '$bulkPass' | realm join -v -U $admin $DOMAIN

			# Needed on Ubuntu
			export DEBIAN_FRONTEND=noninteractive
			pam-auth-update --enable mkhomedir --force

			# Enable getent passwd/group. It may make the system slower with large databases
			sed -i '/^\[domain\/.*]$/a enumerate = true' /etc/sssd/sssd.conf
			systemctl restart sssd
			
			
			#https://wiki.ubuntuusers.de/Howto/Samba-AD_Tipps/

			realm discover $REALM

			echo '$bulkPass' | LDAPTLS_REQCERT=never LANG=C /usr/sbin/adcli join --verbose --domain $DOMAIN --domain-realm $REALM --use-ldaps --domain-controller $IP --computer-name $client --login-type user --login-user $admin --stdin-password

			echo '[nss]
filter_groups = root
filter_users = root
reconnection_retries = 3

[pam]
reconnection_retries = 3

[sssd]
domains = $DOMAIN
config_file_version = 2
services = nss, pam
default_domain_suffix = $REALM

[domain/$DOMAIN]
ad_domain = $DOMAIN
krb5_realm = $REALM
realmd_tags = manages-system joined-with-samba
cache_credentials = True
id_provider = ad
krb5_store_password_if_offline = True
default_shell = /bin/bash
ldap_id_mapping = True
use_fully_qualified_names = True
fallback_homedir = /home/%d/%u
access_provider = ad
auth_provider = ad
chpass_provider = ad
access_provider = ad
ldap_schema = ad
dyndns_update = true
dyndns_refresh_interval = 43200
dyndns_update_ptr = true
dyndns_ttl = 3600
enumerate = true' > /etc/sssd/sssd.conf

		realm permit --all
			
			
		");
	}





/**
**name CLibreWorkspace::addClientToLW($client, $clientIP)
**description Adds an m23 client into Samba's DNS server.
**parameter client: Name of a client.
**parameter clientIP: IP of the client.
**returns true on successfully adding, otherwise false.
**/
	public static function addClientToLW($client, $clientIP)
	{
		$DOMAIN = $adminPassword = $ldap_dc = $REALM = '';
		CLibreWorkspace::parseSettingsFile($DOMAIN, $adminPassword, $ldap_dc, $REALM);
		

		$ret = SERVER_runInBackground('LW_addPosix',
		"
			ret=0
			# Create DNS entry
			samba-tool dns add la.$DOMAIN $DOMAIN $client A $clientIP -U 'administrator%$$adminPassword'
			ret=$[ \$ret + \$? ]

			echo \$ret
		", 'root', false, true);

		// Split the output of the commands into lines
		$lines = explode("\n", $ret);

		// Check all lines if they contain hints for successfully adding
		$ok = (HELPER_str_contains($lines[0], 'Record added successfully')) &&
			($lines[1] == 0);

		return($ok ? true : $ret);
	}





/**
**name CLibreWorkspace::subdomainOrIP()
**description Get m23's subdomain if m23 is installed on Libre Workspace or the IP of the server otherwise.
**returns m23's subdomain if m23 is installed on Libre Workspace or the IP of the server otherwise.
**/
	public static function subdomainOrIP()
	{
		if (CLibreWorkspace::isAvailable() && file_exists('/m23/etc/domain.override'))
			return(file_get_contents('/m23/etc/domain.override'));
		else
			return(getServerIP());
	}
}

?>